@@ -9,12 +9,13 @@
 #include <string.h>
 
 #include "args.h"
+#include "buf_size.h"
 #include "error.h"
+#include "functions/both_libs.h"
 #include "functions/custom_target.h"
 #include "functions/file.h"
 #include "install.h"
 #include "lang/typecheck.h"
-#include "log.h"
 #include "options.h"
 #include "platform/assert.h"
 #include "platform/filesystem.h"
@@ -79,12 +80,12 @@ add_subdirs_includes_iter(struct workspace *wk, void *_ctx, obj val)
 {
 	obj *cflags = _ctx;
 
-	if (str_eql(get_str(wk, val), &WKSTR("."))) {
+	if (str_eql(get_str(wk, val), &STR("."))) {
 		obj_array_push(wk, *cflags, make_str(wk, "-I${includedir}"));
 	} else {
-		SBUF(path);
+		TSTR(path);
 		path_join(wk, &path, "-I${includedir}", get_cstr(wk, val));
-		obj_array_push(wk, *cflags, sbuf_into_str(wk, &path));
+		obj_array_push(wk, *cflags, tstr_into_str(wk, &path));
 	}
 
 	return ir_cont;
@@ -93,7 +94,7 @@ add_subdirs_includes_iter(struct workspace *wk, void *_ctx, obj val)
 static bool
 module_pkgconf_lib_to_lname(struct workspace *wk, obj lib, obj *res)
 {
-	SBUF(basename);
+	TSTR(basename);
 	const char *str;
 
 	switch (get_obj_type(wk, lib)) {
@@ -117,11 +118,11 @@ module_pkgconf_lib_to_lname(struct workspace *wk, obj lib, obj *res)
 		return true;
 	}
 
-	struct str s = WKSTR(str);
-	if (str_startswith(&s, &WKSTR("-l"))) {
+	struct str s = STRL(str);
+	if (str_startswith(&s, &STR("-l"))) {
 		s.len -= 2;
 		s.s += 2;
-	} else if (str_startswith(&s, &WKSTR("lib"))) {
+	} else if (str_startswith(&s, &STR("lib"))) {
 		s.len -= 3;
 		s.s += 3;
 	}
@@ -142,7 +143,7 @@ module_pkgconf_process_reqs_iter(struct workspace *wk, void *_ctx, obj val)
 
 	switch (get_obj_type(wk, val)) {
 	case obj_string: obj_array_push(wk, ctx->dest, val); break;
-	case obj_both_libs: val = get_obj_both_libs(wk, val)->dynamic_lib;
+	case obj_both_libs: val = decay_both_libs(wk, val);
 	/* fallthrough */
 	case obj_build_target: {
 		struct obj_build_target *tgt = get_obj_build_target(wk, val);
@@ -212,7 +213,7 @@ str_to_file_iter(struct workspace *wk, void *_ctx, obj v)
 	obj *arr = _ctx;
 
 	obj f;
-	make_obj(wk, &f, obj_file);
+	f = make_obj(wk, obj_file);
 	*get_obj_file(wk, f) = v;
 	obj_array_push(wk, *arr, f);
 	return ir_cont;
@@ -223,7 +224,7 @@ module_pkgconf_process_libs_iter(struct workspace *wk, void *_ctx, obj val)
 {
 	struct module_pkgconf_process_libs_iter_ctx *ctx = _ctx;
 
-	/* obj_fprintf(wk, log_file(), "%o\n", val); */
+	/* obj_lprintf(wk, "processing lib %o\n", val); */
 
 	switch (get_obj_type(wk, val)) {
 	case obj_string: {
@@ -254,7 +255,7 @@ module_pkgconf_process_libs_iter(struct workspace *wk, void *_ctx, obj val)
 		obj_array_push(wk, ctx->pc->libs[ctx->vis], lib);
 		break;
 	}
-	case obj_both_libs: val = get_obj_both_libs(wk, val)->dynamic_lib;
+	case obj_both_libs: val = decay_both_libs(wk, val);
 	/* fallthrough */
 	case obj_build_target: {
 		struct obj_build_target *tgt = get_obj_build_target(wk, val);
@@ -271,7 +272,7 @@ module_pkgconf_process_libs_iter(struct workspace *wk, void *_ctx, obj val)
 					    ctx->err_node,
 					    tgt->dep.raw.deps,
 					    ctx->pc,
-					    pkgconf_visibility_priv,
+					    pkgconf_visibility_pub,
 					    false)) {
 					return ir_err;
 				}
@@ -318,6 +319,7 @@ module_pkgconf_process_libs_iter(struct workspace *wk, void *_ctx, obj val)
 		}
 
 		switch (dep->type) {
+		case dependency_type_system:
 		case dependency_type_declared: {
 			// TODO: I'm pretty sure this doesn't obey partial
 			// dependency semantics if this is a sub dependency of
@@ -361,18 +363,25 @@ module_pkgconf_process_libs_iter(struct workspace *wk, void *_ctx, obj val)
 			break;
 		case dependency_type_external_library: {
 			obj link_with_files;
-			make_obj(wk, &link_with_files, obj_array);
-			obj_array_foreach(wk, dep->dep.link_with, &link_with_files, str_to_file_iter);
+			link_with_files = make_obj(wk, obj_array);
 
-			if (!module_pkgconf_process_libs(
-				    wk, ctx->err_node, link_with_files, ctx->pc, ctx->vis, false)) {
-				return ir_err;
+			if (dep->dep.link_with) {
+				obj_array_foreach(wk, dep->dep.link_with, &link_with_files, str_to_file_iter);
+
+				if (!module_pkgconf_process_libs(
+						wk, ctx->err_node, link_with_files, ctx->pc, ctx->vis, false)) {
+					return ir_err;
+				}
+			}
+
+			if (dep->dep.link_with_not_found) {
+				if (!module_pkgconf_process_libs(
+						wk, ctx->err_node, dep->dep.link_with_not_found, ctx->pc, ctx->vis, false)) {
+					return ir_err;
+				}
 			}
 			break;
 		}
-		case dependency_type_appleframeworks:
-			// TODO: actually add correct -framework arguments
-			break;
 		case dependency_type_not_found: break;
 		}
 		break;
@@ -385,9 +394,9 @@ module_pkgconf_process_libs_iter(struct workspace *wk, void *_ctx, obj val)
 
 		struct obj_custom_target *tgt = get_obj_custom_target(wk, val);
 		obj out;
-		obj_array_index(wk, tgt->output, 0, &out);
+		out = obj_array_index(wk, tgt->output, 0);
 
-		if (str_endswith(get_str(wk, *get_obj_file(wk, out)), &WKSTR(".a"))) {
+		if (str_endswith(get_str(wk, *get_obj_file(wk, out)), &STR(".a"))) {
 			return ir_cont;
 		}
 
@@ -450,14 +459,14 @@ module_pkgconf_declare_var(struct workspace *wk,
 		const char *reserved[] = { "prefix", "libdir", "includedir", NULL };
 		uint32_t i;
 		for (i = 0; reserved[i]; ++i) {
-			if (str_eql(key, &WKSTR(reserved[i]))) {
+			if (str_eql(key, &STRL(reserved[i]))) {
 				vm_error_at(wk, err_node, "variable %s is reserved", reserved[i]);
 				return false;
 			}
 		}
 
 		for (i = 0; i < ARRAY_LEN(module_pkgconf_diropts); ++i) {
-			if (str_eql(key, &WKSTR(module_pkgconf_diropts[i].optname))) {
+			if (str_eql(key, &STRL(module_pkgconf_diropts[i].optname))) {
 				module_pkgconf_diropts[i].added = true;
 			}
 
@@ -465,14 +474,14 @@ module_pkgconf_declare_var(struct workspace *wk,
 				continue;
 			}
 
-			if (str_startswith(val, &WKSTR(module_pkgconf_diropts[i].name))) {
+			if (str_startswith(val, &STRL(module_pkgconf_diropts[i].name))) {
 				module_pkgconf_diropts[module_pkgconf_diropt_prefix].refd = true; // prefix
 				module_pkgconf_diropts[i].refd = true;
 			}
 		}
 	}
 
-	SBUF(esc);
+	TSTR(esc);
 	const char *esc_val;
 	if (escape) {
 		pkgconf_escape(wk, &esc, val->s);
@@ -496,7 +505,7 @@ module_pkgconf_declare_builtin_dir_var(struct workspace *wk, const char *opt, ob
 		valstr = make_strf(wk, "${prefix}/%s", get_cstr(wk, val));
 	}
 
-	module_pkgconf_declare_var(wk, 0, true, true, &WKSTR(opt), get_str(wk, valstr), dest);
+	module_pkgconf_declare_var(wk, 0, true, true, &STRL(opt), get_str(wk, valstr), dest);
 }
 
 struct module_pkgconf_process_vars_ctx {
@@ -581,7 +590,7 @@ module_pkgconf_prepend_libdir(struct workspace *wk, struct args_kw *install_dir_
 	obj libdir;
 	const char *path;
 	if (install_dir_opt->set) {
-		SBUF(rel);
+		TSTR(rel);
 		obj pre;
 		get_option_value(wk, current_project(wk), "prefix", &pre);
 
@@ -603,7 +612,7 @@ module_pkgconf_prepend_libdir(struct workspace *wk, struct args_kw *install_dir_
 	}
 
 	obj arr;
-	make_obj(wk, &arr, obj_array);
+	arr = make_obj(wk, obj_array);
 	obj_array_push(wk, arr, libdir);
 	obj_array_extend_nodup(wk, arr, *libs);
 	*libs = arr;
@@ -633,7 +642,7 @@ static void
 module_pkgconf_remove_dups(struct workspace *wk, obj *list, obj exclude)
 {
 	obj arr;
-	make_obj(wk, &arr, obj_array);
+	arr = make_obj(wk, obj_array);
 
 	struct module_pkgconf_remove_dups_ctx ctx = {
 		.exclude = exclude,
@@ -780,19 +789,19 @@ func_module_pkgconfig_generate(struct workspace *wk, obj self, obj *res)
 		.dataonly = akw[kw_dataonly].set ? get_obj_bool(wk, akw[kw_dataonly].val) : false,
 	};
 	for (i = 0; i < 2; ++i) {
-		make_obj(wk, &pc.libs[i], obj_array);
-		make_obj(wk, &pc.reqs[i], obj_array);
+		pc.libs[i] = make_obj(wk, obj_array);
+		pc.reqs[i] = make_obj(wk, obj_array);
 	}
-	make_obj(wk, &pc.cflags, obj_array);
-	make_obj(wk, &pc.variables, obj_array);
-	make_obj(wk, &pc.builtin_dir_variables, obj_array);
-	make_obj(wk, &pc.exclude, obj_array);
+	pc.cflags = make_obj(wk, obj_array);
+	pc.variables = make_obj(wk, obj_array);
+	pc.builtin_dir_variables = make_obj(wk, obj_array);
+	pc.exclude = make_obj(wk, obj_array);
 
 	obj mainlib = 0;
 	if (an[0].set) {
 		switch (get_obj_type(wk, an[0].val)) {
 		case obj_both_libs: {
-			mainlib = get_obj_both_libs(wk, an[0].val)->dynamic_lib;
+			mainlib = decay_both_libs(wk, an[0].val);
 			break;
 		}
 		case obj_build_target: mainlib = an[0].val; break;
@@ -930,9 +939,9 @@ func_module_pkgconfig_generate(struct workspace *wk, obj self, obj *res)
 		filebase = akw[kw_filebase].val;
 	}
 
-	SBUF(path);
+	TSTR(path);
 	path_join(wk, &path, wk->muon_private, get_cstr(wk, filebase));
-	sbuf_pushs(wk, &path, ".pc");
+	tstr_pushs(wk, &path, ".pc");
 
 	if (!module_pkgconf_write(wk, path.buf, &pc)) {
 		return false;
@@ -942,11 +951,11 @@ func_module_pkgconfig_generate(struct workspace *wk, obj self, obj *res)
 		get_obj_build_target(wk, mainlib)->generated_pc = filebase;
 	}
 
-	make_obj(wk, res, obj_file);
-	*get_obj_file(wk, *res) = sbuf_into_str(wk, &path);
+	*res = make_obj(wk, obj_file);
+	*get_obj_file(wk, *res) = tstr_into_str(wk, &path);
 
 	{
-		SBUF(install_dir_buf);
+		TSTR(install_dir_buf);
 		const char *install_dir;
 		if (akw[kw_install_dir].set) {
 			install_dir = get_cstr(wk, akw[kw_install_dir].val);
@@ -962,11 +971,11 @@ func_module_pkgconfig_generate(struct workspace *wk, obj self, obj *res)
 			install_dir = install_dir_buf.buf;
 		}
 
-		SBUF(dest);
+		TSTR(dest);
 		path_join(wk, &dest, install_dir, get_cstr(wk, filebase));
-		sbuf_pushs(wk, &dest, ".pc");
+		tstr_pushs(wk, &dest, ".pc");
 
-		push_install_target(wk, *get_obj_file(wk, *res), sbuf_into_str(wk, &dest), 0);
+		push_install_target(wk, *get_obj_file(wk, *res), tstr_into_str(wk, &dest), 0);
 	}
 
 	return true;
